-- This Source Code Form is subject to the terms of the bCDDL, v. 1.1.
-- If a copy of the bCDDL was not distributed with this
-- file, You can obtain one at http://beamng.com/bCDDL-1.1.txt

local M = {}

local frontGearHydro = nil
local leftGearHydro = nil
local rightGearHydro = nil

local mainClosedThreshold = 0.552
local frontClosedThreshold = 0.29

local gds = 3.5
local mainSmoother = newTemporalSmoothingNonLinear(gds, gds, 1)
local mainFastSmoother = newTemporalSmoothingNonLinear(gds, gds * 3, 1)
local frontSmoother = newTemporalSmoothingNonLinear(gds, gds, 1)
local frontClosingSmoother = newTemporalSmoothingNonLinear(gds, gds, 0)
local mainClosingSmoother = newTemporalSmoothingNonLinear(gds, gds, 0)

local function init()
  --nil beams just in case
  frontGearHydro = nil
  leftGearHydro  = nil
  rightGearHydro = nil
  
  --init electrics
  electrics.values["geardoor_F"] = 0
  electrics.values["geardoor_M"] = 0
  electrics.values["geardoor_Mfast"] = 0
  electrics.values["geardoor_F_closing"] = 0
  electrics.values["geardoor_M_closing"] = 0
  electrics.values["gear_L_extent"] = 0
  electrics.values["gear_R_extent"] = 0
  electrics.values["gear_F_extent"] = 0
  electrics.values["gear_L_open"] = 0
  electrics.values["gear_R_open"] = 0
  electrics.values["gear_F_open"] = 0
  
  --get beams
  for k,beam in pairs(v.data.beams) do
    if beam.name ~= nil then
      --assign things
      if beam.name == "frontgear" then frontGearHydro =   beam end
      if beam.name == "maingear_L" then leftGearHydro = beam end
      if beam.name == "maingear_R" then rightGearHydro = beam end
    end
  end
end

local function map(x, in_min, in_max, out_min, out_max)
  return (x - in_min) * (out_max - out_min) / (in_max - in_min) + out_min
end

local function updateGFX(dt)
  --get gear extents for gear door stuff
  if leftGearHydro ~= nil then
    local leftLength = obj:getBeamLengthRefRatio(leftGearHydro.cid)
    local leftFactor = math.max(map(leftLength, mainClosedThreshold, 1, 0, 1), 0)
    electrics.values["gear_L_extent"] = leftFactor
  end

  if rightGearHydro ~= nil then
    local rightLength = obj:getBeamLengthRefRatio(rightGearHydro.cid)
    local rightFactor = math.max(map(rightLength, mainClosedThreshold, 1, 0, 1), 0)
    electrics.values["gear_R_extent"] = rightFactor
  end
  
  if frontGearHydro ~= nil then
    local frontLength = obj:getBeamLengthRefRatio(frontGearHydro.cid)
    local frontFactor = math.max(map(frontLength, frontClosedThreshold, 1, 0, 1), 0)
    electrics.values["gear_F_extent"] = frontFactor
  end
  
  if electrics.values["gear_R_extent"] >= 1 then electrics.values["gear_R_open"] = 1 else electrics.values["gear_R_open"] = 0 end
  if electrics.values["gear_L_extent"] >= 1 then electrics.values["gear_L_open"] = 1 else electrics.values["gear_L_open"] = 0 end
  if electrics.values["gear_F_extent"] >= 1 then electrics.values["gear_F_open"] = 1 else electrics.values["gear_F_open"] = 0 end
  
  --calculate the rest
  if leftGearHydro ~= nil or rightGearHydro ~= nil then
    local beam = leftGearHydro or rightGearHydro
    local mainLength = obj:getBeamLengthRefRatio(beam.cid)
    
    --set electrics
    if mainLength < mainClosedThreshold then
      electrics.values["geardoor_M"] = mainSmoother:get(0, dt)
      electrics.values["geardoor_Mfast"] = mainFastSmoother:get(0, dt)
    else
      electrics.values["geardoor_Mfast"] = mainFastSmoother:get(1, dt)
      electrics.values["geardoor_M"] = mainSmoother:get(1, dt)
    end
    
    if mainLength < 1 and mainLength > mainClosedThreshold then
      electrics.values["geardoor_M_closing"] = mainClosingSmoother:get(1, dt)
    else
      electrics.values["geardoor_M_closing"] = mainClosingSmoother:get(0, dt)
    end
  end
  
  if frontGearHydro ~= nil then
    local frontLength = obj:getBeamLengthRefRatio(frontGearHydro.cid)
    --set electrics
    if frontLength < frontClosedThreshold then
      electrics.values["geardoor_F"] = frontSmoother:get(0, dt)
    else
      electrics.values["geardoor_F"] = frontSmoother:get(1, dt)
    end
    
    if frontLength < 1 and frontLength > frontClosedThreshold then
      electrics.values["geardoor_F_closing"] = frontClosingSmoother:get(1, dt)
    else
      electrics.values["geardoor_F_closing"] = frontClosingSmoother:get(0, dt)
    end
  end
  
end

-- public interface
M.updateGFX      = updateGFX
M.onInit      = init
M.onReset     = init
return M